
# q-expansion of newform 4800.2.a.bf, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 4800
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [-1], [0], [4], [-4], [0], [-4], [0], [4], [6], [-4], [-8], [-10], [-4], [-4], [-12], [4], [-2], [4], [0], [8], [12], [-4], [-10], [-8], [2], [-4], [-12], [2], [-12], [4], [12], [-12], [16], [2], [20], [8], [-20], [12], [-4], [4], [10], [-24], [-16], [-4], [12], [0], [-20], [20], [-26], [-20], [-8], [-2], [-12], [-12], [4], [-6], [-4], [-32], [6], [28], [12], [-12], [-24], [-16], [4], [-8], [8], [-12], [14], [28], [-24], [4], [24], [8], [-28], [34], [-8], [-14], [26], [-28], [-10], [-8], [-16], [12], [-36], [18], [-40], [-6], [12], [-12], [16], [-12], [-36], [24], [36], [-42], [-38], [28], [2], [-28], [-12], [-20], [26], [40], [32], [36], [36], [24], [38], [-28], [-8], [44], [-16], [28], [14], [-36], [12], [4], [28], [22], [0], [4], [44], [-32], [-26], [6], [-8], [-12], [-16], [40], [36], [28], [16], [42], [-18], [-12], [-28], [28], [6], [40], [14], [12], [-28], [18], [56], [8], [12], [-40], [-12], [56], [30], [12], [-36], [-12], [-48], [12], [18], [-32], [10], [-12], [4], [20], [-12], [12], [-28], [-28], [-24], [-34], [-36], [-12], [18], [-32], [-8], [-4], [-42], [-16], [-30], [-20], [-14], [-44], [-44], [32], [36], [20], [30], [16], [28], [-22], [56], [-24], [44], [16], [6], [-12], [36], [-18], [0], [36], [20], [54], [36], [48], [46], [-4], [-36], [36], [-4], [-26], [-32], [-32], [30], [44], [36], [-16], [70], [68], [-14], [-52], [4], [-42], [-4], [30], [-4], [-60], [-54], [-28], [8], [68], [20], [-16], [40], [-20], [-42], [-52], [76], [-30], [44], [12], [48], [-36], [8], [28], [62], [44], [-48], [-60], [36], [-40], [52], [-48], [2], [-52], [58], [-28], [12], [-22], [-44], [20], [64], [-36], [-28], [38], [-40], [36], [8], [-42], [-22], [-4], [-12], [34], [4], [-56], [44], [-40], [-52], [4], [-50], [38], [12], [36], [28], [-4], [-22], [4], [-8], [24], [-76], [4], [-46], [-26], [-12], [60], [36], [-8], [-6], [-60], [60], [44], [36], [-32], [-12], [52], [-84], [-64], [8], [4], [-50], [-64], [-72], [36], [30], [-30], [60], [76], [38], [-44], [32], [-40], [66], [-40], [-32], [6], [76], [12], [14], [24], [12], [-20], [-68], [-34], [4], [-44], [-20], [48], [36], [2], [-84], [22], [52], [-24], [-20], [82], [28], [-44], [12], [-42], [4], [80], [12], [-40], [-24], [54], [12], [70], [12], [48], [-12], [20], [-92], [-8], [-6], [60], [92], [-60], [80], [-12], [12], [-90], [-20], [56], [-44], [18], [-12], [0], [44], [32], [48], [-30], [40], [-54], [-44], [-76], [-68], [24], [-12], [12], [-64], [12], [-68], [-46], [36], [-60], [52], [80], [-88], [-20], [-26], [32], [78], [2], [20], [-28], [-60], [66], [76], [-48], [18], [28], [36], [-32], [-84], [-36], [-80], [-40], [10], [16], [-12], [-28], [-12], [-6], [56], [76], [-108], [64], [12], [28], [58], [0], [40]]
